#!/usr/local/bin/perl5

require "configure.pl";
require "cgi-lib.pl";
require "utilities.pl";
require "database.pl";
require "summary.pl";

#
#  Output the report page.
#

sub report_page {
    local ($pc_member, $id) = @_;

    local ($pid, $type, $title, $authors) = &get_paper($id);

    if (!defined($title)) {
	print "A paper with the number you entered " .
	      "($id) could not be found.<BR>\n";
	die;
	}

    if (&authored_paper($pc_member, $id)) {
	print "Paper $id: <I>$title</I>, by $authors\n";
        print "<P>It looks like you ($pc_member) are an author\n";
	print "of the paper, so you cannot file a report.\n";
        print "<P>If you are not an author, contact $Admin.\n";
	&log_access($pc_member, "report, $id", "access denied");
        }	

    else {

        if (!&assigned_paper($pc_member, $id)) {
            print "<BLINK><FONT COLOR=red> This is a voluntary report, " .
              "because you were not assigned this paper.</FONT></BLINK><HR>\n";
            }

	local ($ptype) = $type eq "regular" ? "" : " ($type)";

	&log_access($pc_member, "report, $id", "ok");

############################################ vvvvvvvvvvvvvvv
        print <<END_OF_PAGE;				

<CENTER><H1>$Conference Report Form</H1></CENTER>
Paper <STRONG>$id</STRONG>$ptype:
<I>$title</I><BR>
Author(s): $authors

<FORM METHOD="POST" ACTION="referee-form.pl">

<INPUT type="hidden" NAME="pc_member" VALUE="$pc_member">
<INPUT type="hidden" NAME="paper_id" VALUE="$id">

(Greater numbers are better ratings.)

<BR>
Relevance to $Conference:
<SELECT NAME="appropriateness">
<OPTION> 4 <OPTION> 3 <OPTION> 2 <OPTION> 1 <OPTION SELECTED> ?
</SELECT>

<BR>
Significance:
<SELECT NAME="significance">
<OPTION> 4 <OPTION> 3 <OPTION> 2 <OPTION> 1 <OPTION SELECTED> ?
</SELECT>

<BR>
Originality:
<SELECT NAME="originality">
<OPTION> 4 <OPTION> 3 <OPTION> 2 <OPTION> 1 <OPTION SELECTED> ?
</SELECT>

<BR>
Presentation:
<SELECT NAME="presentation">
<OPTION> 4 <OPTION> 3 <OPTION> 2 <OPTION> 1 <OPTION SELECTED> ?
</SELECT>

<BR>
Referee's confidence:
<SELECT NAME="confidence">
<OPTION> 4 <OPTION> 3 <OPTION> 2 <OPTION> 1 <Option SELECTED> ?
</SELECT>

<BR>
Overall rating:
<SELECT NAME="overall">
<OPTION> Strong Accept
<OPTION> Accept
<OPTION> Weak Accept
<OPTION SELECTED> No Opinion
<OPTION> Weak Reject
<OPTION> Reject
<OPTION> Strong Reject
</SELECT>

<P>
Comments for Authors (required):<BR>
<TEXTAREA NAME="comments" ROWS=20 COLS=70></TEXTAREA>

<P>
<HR SIZE=5>

<P>
Comments for Program Committee Only (optional):<BR>
<TEXTAREA NAME="comments_pc" ROWS=4 COLS=70></TEXTAREA>

<P>
PC Member:
<STRONG>$pc_member</STRONG>
<BR>
Referee (if different from PC member):
<INPUT TYPE="text" SIZE=30 NAME="referee">

<HR SIZE=5>

<CENTER>
<FONT SIZE=4><INPUT TYPE="submit" VALUE="Submit Report"></FONT>
<FONT SIZE=2><INPUT TYPE="reset" VALUE="Reset Form"></FONT>
</CENTER>
</FORM>

END_OF_PAGE
############################################ ^^^^^^^^^^^^^^^^

	}
    }  # report_page

#
#  Output the discussion page.
#

sub discussion_page {
    local ($pc_member, $id) = @_;

    if (!$Discuss_enabled && !$root) {
        print "Discussion pages are disabled."; 
	die;
        }

    local ($pid, $type, $title, $authors) = &get_paper($id);

    if (!defined($title)) {
	print "A paper with the number you entered " .
	      "($id) could not be found.<BR>\n";
	die;
	}

    if (&authored_paper($pc_member, $id)) {
	print "Paper $id: <I>$title</I>, by $authors\n";
        print "<P>Access denied, because it looks like you ($pc_member)\n";
	print "are an author of the paper.\n";
        print "<P>If you are not an author, contact $Admin.\n";
	&log_access($pc_member, "discussion, $id", "access denied");
        }	
    else {

	local (%mandatory, %voluntary, %current_reports, %aa);
	local (@records, @accepts, @rejects);
	local ($ptype, $referee, $rating);
	local ($record, $contributor, $event, $date, $comm);

	&log_access($pc_member, "discussion, $id", "ok");

	$ptype = $type eq "regular" ? "" : " ($type)";

	print "<H1>Discussion Page for Paper $id$ptype</H1>\n";
	print "Title: <I>$title</I><BR>\n";
	print "Author(s): $authors\n";
	print "<P> - <A HREF=\"$FTP_abstract_dir/$id.txt\">Abstract</A> - ";
	print "<A HREF=\"$FTP_ps_dir/$id.ps.gz\">";
        print "FTP the whole paper now</A> -\n";

	print "<P>On this page:<P> -\n";
	print "<A HREF=\"#summary\">Summary</A> - \n";
	print "<A HREF=\"#reports\">Reports</A> - \n";
	print "<A HREF=\"#discussion\">Discussion</A> - \n";
	print "<A HREF=\"#discuss_form\">Comments Form</A> - \n";
	print "<A HREF=\"#vote_form\">Voting Form</A> - \n";

	print "<HR>\n";

	#
	# Summary
	#

	print "<H2><A NAME=\"summary\">Summary of Responses</A></H2>\n";
        print "(Links take you down to the corresponding reports.)\n";

        &get_overalls($id,*mandatory,*voluntary);

	print "<P>Mandatory Reports\n";
	print "<UL>\n";
	foreach $referee (keys(%mandatory)) {
	    $rating = $mandatory{$referee};
	    if ($rating =~ /no report/) {
		print "<LI>$referee: $rating\n";
	        }
	    else {
		print "<LI><A HREF=\"#$referee\">$referee: $rating</A>\n";
	        }
	    }
	print "</UL>\n";

	print "Voluntary Reports\n";

	print "<UL>\n";
	if (!keys(%voluntary)) {print "<LI>(none)\n<P>";}
	foreach $referee (keys(%voluntary)) {
	    $rating = $voluntary{$referee};
	    print "<LI><A HREF=\"#$referee\">$referee: $rating</A>\n";
	    }
	print "</UL>\n";

	&get_votes($id, *accepts, *rejects);

	print "Current Voting (including automatic votes from reports,\n";
	print "unless superceded by explicit votes)\n";
	print "<UL>\n";
	print "<LI> Accept: ", scalar(@accepts), "   (@accepts)\n";
	print "<LI> Reject: ", scalar(@rejects), "   (@rejects)\n";
	print "</UL>\n";
	print "<HR>\n";

        print "<H2><A NAME=\"reports\">Current reports on paper $id</A></H2>\n";
	print "This list does not contain reports that have\n";
	print "been superseded by more recent reports\n";
	print "from the same PC member.\n<P>";

	@records = &retrieve_events($Report_file, "paper_id", $id);

	if (@records == 0) {print "(none)\n<HR>\n";}

	#
	# Reports
	#
	# We shall print only the latest report for each pc member.
	# Build an associative array of reports, indexed by pc member.
	# Since reports are ordered by event_id (i.e., time), any
	# obsolete reports will bw overwritten by the current report.
	#

	foreach $record (@records) {
	    %aa = &record_to_assoc_array(*record);
	    $referee = $aa{"pc_member"};
	    $current_reports{$referee} = $record;
	    }

	#
	# OK, we now have the current reports;  print them.
	#

	foreach $referee (keys(%current_reports)) {
	    $record = $current_reports{$referee};
	    %aa = &record_to_assoc_array(*record);
	    $event = $aa{"event_id"};
	    $date  = $aa{"date"};
	    print "<A NAME=\"$referee\">Report</A>\n";
	    print "(PC member $referee, event $event) $date\n";
	    print "<PRE>\n";
	    &print_ref_report(STDOUT, *aa);
	    print "</PRE>\n";
	    print "<HR>\n";
	    }
	
	#
	# Print all of the discussion
	#

        print "<H2><A NAME=\"discussion\">Discussion on paper $id</A></H2>\n";
	@records = &retrieve_events($Discuss_file, "paper_id", $id);
	if (@records == 0) {print "(none)\n<HR>\n";}
	foreach $record (@records) {
	    %aa = &record_to_assoc_array(*record);
	    $contributor = $aa{"pc_member"};
	    $event  = $aa{"event_id"};
	    $date   = $aa{"date"};
	    $comm   = $aa{"comments"};
	    print "From $contributor (event $event) $date\n";
	    print "<PRE>\n";
	    print $comm;
	    print "</PRE>\n";
	    print "<HR>\n";
            }

	#
	# Print the discussion form and the voting form
	#

############################################ vvvvvvvvvvvvvvv
	print <<END_OF_TAIL

<FORM METHOD="POST" ACTION="discuss-form.pl">

<INPUT type="hidden" NAME="pc_member" VALUE="$pc_member">
<INPUT type="hidden" NAME="paper_id" VALUE="$id">

<H3><A NAME="discuss_form">Comments Form</A></H3>

Paper $id$ptype: <I>$title</I>.
<P>
Add to the discussion:<BR>
<TEXTAREA NAME="comments" ROWS=15 COLS=60></TEXTAREA>

<BR>
<INPUT TYPE="submit" VALUE="Submit Comments">
<FONT SIZE=2><INPUT TYPE="reset" VALUE="Clear Form"></FONT>

</FORM>

<HR>
<FORM METHOD="POST" ACTION="vote-form.pl">

<INPUT type="hidden" NAME="pc_member" VALUE="$pc_member">
<INPUT type="hidden" NAME="paper_id" VALUE="$id">

<H3><A NAME="vote_form">Voting Form</A></H3>

Paper $id$ptype: <I>$title</I>.

<P>
If you file a full report on a paper, a vote will
be cast automatically, based on your overall rating.
You can change your vote later by simply filing another (complete) report.
Casting an explicit vote here will not change the rating
in your report.
Explicit votes are intended for PC members who have not
filed reports.  (But it is expected that they will have looked
at the paper and followed the discussion).  

<P>
<INPUT type="radio" NAME="vote" VALUE="accept"> Accept<BR>
<INPUT type="radio" NAME="vote" VALUE="reject"> Reject

<BR>
<INPUT TYPE="submit" VALUE="Cast your vote now">

</FORM>
<HR>

END_OF_TAIL
############################################ ^^^^^^^^^^^^^^^^

	}
    }  # discussion_page

#
#  list_work_tut - workshops and tutorials
#

sub list_work_tut {

    &log_access($pc_member, "list work_tut", "ok");

    print `cat $Work_tut_file`;

    }  # list_work_tut

#
#  list_papers - with links to FTP abstracts and PostScript.
#

sub list_papers {

    &log_access($pc_member, "list papers", "ok");

    print "<H1>This is the list of papers submitted to $Conference.</H1>\n";
    print "Click on the paper ID to FTP the abstract (plain text) .\n<BR>";
    print "Click on the paper title to FTP the whole paper (PostScript).\n";
    print "<HR>\n";
    if (!open(FH, $File_of_papers)) {
        print "Cannot open file $File_of_papers: $!";
        die;
        }
    print "<UL>\n";

    local ($line, $id, $type, $title, $authors, $ptype);

    while ($line = <FH>) {
	chop($line);
	($id,$type,$title,$authors) = split(/\s+#\s+/, $line);
	$ptype = $type eq "regular" ? "" : " ($type)";
	print "<LI><A HREF=\"$FTP_abstract_dir/$id.txt\">$id$ptype</A>.\n";
        print "<A HREF=\"$FTP_ps_dir/$id.ps.gz\"><I>$title</I></A>,\n";
        print "$authors<BR>\n";
	}
    print "</UL>\n";
    close(FH);
    print "<HR>\n";
    }  # list_papers

#
# member_data
#

sub member_data {
    local ($pc_member) = @_;

    &log_access($pc_member, "member data", "ok");

    local ($date) = &chop_date;

    print "<H1>PC Member $pc_member</H1>\n";
    print "This page contains all of the reports, discussion,\n";
    print "and votes you have filed up to $date.\n";
    print "<HR>\n";

    local (@authored_papers, $id, $pid, $type, $title, $authors, $ptype);
    @authored_papers = &get_authored_papers($pc_member);

    print "You are an author of the following papers:\n";
    print "<UL>\n";
    if (@authored_papers == 0) {print "(none)\n";}
    foreach $id (@authored_papers) {
	($pid, $type, $title, $authors) = &get_paper($id);
	($ptype) = $type eq "regular" ? "" : " ($type)";
	print "<LI>Paper $id$ptype: <I>$title</I>, $authors.\n";
        }
    print "</UL>\n";
    print "<HR>\n";

    local (@assigned_papers, $id, $pid, $type, $title, $authors);
    local (@reports, $report, %aa, $rating, $event, $count, $rid);

    @assigned_papers = &get_assigned_papers($pc_member);

    @reports = &retrieve_events($Report_file, "pc_member", $pc_member);

    print "You are supposed to referee the following papers:\n";
    print "<UL>\n";
    if (@assigned_papers == 0) {print "(none)\n";}
    foreach $id (@assigned_papers) {
	($pid, $type, $title, $authors) = &get_paper($id);
	($ptype) = $type eq "regular" ? "" : " ($type)";
	print "<LI>Paper $id$ptype: <I>$title</I>, $authors.<BR>\n";

	$count = 0;
	foreach $report (@reports) {
	    %aa = &record_to_assoc_array(*report);
	    $rid = $aa{"paper_id"};
	    if ($rid eq $id) {
		$count += 1;
		$rating = $aa{"overall"};
		$event = $aa{"event_id"};
		print " <B>$rating</B> (event $event).";
		print "\n";
	        }
	    $event = $aa{"event_id"};
	    $rating = $aa{"overall"};
            }
        if ($count == 0) {
	    print "No report yet.\n";
	    }
        }
    print "</UL>\n";
    print "<HR>\n";

    local (@records, $record, %aa, $event, $date);

    print "<H2>Reports</H2>\n";
    print "Here are all of the reports you have filed.\n<P>\n";
    if (@reports == 0) {print "(none)\n<HR>";}
    foreach $report (@reports) {
	%aa = &record_to_assoc_array(*report);
	$event = $aa{"event_id"};
	$date  = $aa{"date"};
	print "Report (event $event) $date\n";
	print "<PRE>\n";
	&print_ref_report(STDOUT, *aa);
	print "</PRE>\n";
	print "<HR>\n";
	}

    local (@records, $record, %aa, $pid, $event, $date, $comm);

    print "<H2>Discussion</H2>\n";
    @records = &retrieve_events($Discuss_file, "pc_member", $pc_member);
    if (@records == 0) {print "(none)\n<HR>";}
    foreach $record (@records) {
	%aa = &record_to_assoc_array(*record);
	$pid   = $aa{"paper_id"};
	$event = $aa{"event_id"};
	$date  = $aa{"date"};
	$comm  = $aa{"comments"};
	print "Discuss paper $pid (event $event) $date\n";
	print "<PRE>\n";
	print $comm;
	print "</PRE>\n";
	print "<HR>\n";
	}

    local (@records, $record, %aa, $pid, $event, $date, $vote, $vtype);

    print "<H2>Voting</H2>\n";
    @records = &retrieve_events($Vote_file, "pc_member", $pc_member);
    if (@records == 0) {print "(none)\n<HR>";}
    foreach $record (@records) {
	%aa = &record_to_assoc_array(*record);
	$pid   = $aa{"paper_id"};
	$event = $aa{"event_id"};
	$date  = $aa{"date"};
	$vote  = $aa{"vote"};
	$vtype = $aa{"type"};
	print "Vote <I>$vote</I> ($vtype) on paper $pid (event $event) $date\n";
	print "<HR>\n";
	}
    }  # member_data

####################
print "Content-type: text/html\n\n";      # Print HTML header
####################

# &print_sys_info;                        # Print System info for debugging

&ReadParse(*cgi_input);
# &print_cgi_input(*cgi_input);           # Print CGI input for debugging

$pc_member = &get_cgi_value(*cgi_input, "pc_member");

$transmitted_pwd = $cgi_input{"pet"};
$transmitted_pwd =~ s/\s//g;                    # get rid of any white space.

$encrypted_password = &get_password($pc_member);

$root = 0;

if (!$Password_required) {
    print "<H3><I>Warning: password was not checked.</I></H3>\n";
    }

elsif (crypt($transmitted_pwd, "AR") eq $Root_password_encrypted) {
    print "<H3><BLINK>Message sent to $Admin about root access.</BLINK></H3>\n";
    $root = 1;
    }

elsif ($encrypted_password ne crypt($transmitted_pwd, "AR")) {
    print "<H1>Password is incorrect.</H1>\n";
    die;
    }

$thing_to_do = &get_cgi_value(*cgi_input, "thing_to_do");

if ($thing_to_do eq "list_papers") {
    &list_papers;
    }

elsif ($thing_to_do eq "summary_papers") {
    &summary_papers($pc_member, $root);
    }

elsif ($thing_to_do eq "summary_member") {
    &summary_papers($pc_member, $root, "member_only");
    }

elsif ($thing_to_do eq "list_work_tut") {
    &list_work_tut;
    }

elsif ($thing_to_do eq "submit_report") {
    $id = &get_cgi_value(*cgi_input, "paper_to_referee");
    $id =~ s/\s//g;   # get rid of any white space.

    &report_page($pc_member, $id);
	
    }

elsif ($thing_to_do eq "discuss_paper") {
    $id = &get_cgi_value(*cgi_input, "paper_to_discuss");
    $id =~ s/\s//g;   # get rid of any white space.

    &discussion_page($pc_member, $id);
    }

elsif ($thing_to_do eq "summary_all") {
    print "summary_all: not ready<BR>\n";
    }

elsif ($thing_to_do eq "member_data") {

    &member_data($pc_member);

    }

else {
    print "Error, don't know how to $thing_to_do.<BR>\n";
    }

1; #return true 

