#!/usr/local/bin/perl5

require "configure.pl";
require "cgi-lib.pl";
require "utilities.pl";
require "database.pl";

#
# Map overall ratings to integers
#

%Omap = ("Strong Reject", -3, "Reject", -2, "Weak Reject", -1,
         "Strong Accept",  3, "Accept",  2, "Weak Accept",  1,
	 "No Opinion", 0);

#
# Calculate score of a paper.
#

sub score {
    local ($man, $vol, $acc, $rej) = @_;
    local (@m, $sum, $n);
    @m = split(',', $man);
    $sum = 0;
    foreach $n (@m) { $sum += $n; }
    return $sum;
}  # score

#
# Routine to compare scores of two papers.
# Used in sorting IDs by_score.
# Another perl wierdism -- pseudo-built-in variables $a and $b for sorting.
# (Who knows if they might conflict with other variables named $a or $b.)
#

sub by_score {
    if ($sco{$a} == $sco{$b})
        { return ($a <=> $b); }  # if scores same, smaller ID first
    else
        { return ($sco{$b} <=> $sco{$a}); }  # higher scores first
}  # by_score

#
#  summary_papers
#

sub summary_papers {

    local ($pc_member, $root, $member_only) = @_;

    if (!$Summary_enabled && !$root) {
        print "Summary page is disabled " .
              "while I update the status.<P>-- Bill\n";
	die;
        }

    &log_access($pc_member, "summary papers $member_only", "ok");

    local (%assigned, %authored);	

    #
    # Look in PC_data_file to get assigned and authored papers
    # for each pc member.
    #

    if (! open(FH, $PC_data_file)) {
	print "<H1>Cannot open file $PC_data_file.</H1>\n";
	die;
	}
    while ($line = <FH>) {
        chop($line);
	local ($mem, $passwd, $authored, $assigned) = split("#", $line);
	$mem =~ s/\s//g;  # dump white space
	$assigned{$mem} = $assigned;
	$authored{$mem} = $authored;
	}
    close(FH);

    #
    # Look in Status_file to get status (accept, undecided, etc.)
    # for papers (no entry means undecided).
    #

    local (%sta, $color);

    if (! open(FH, $Status_file)) {
	print "<H1>Cannot open file $Status_file.</H1>\n";
	die;
	}
    while ($line = <FH>) {
        chop($line);
	local ($id, $status) = split("#", $line);
	$id =~ s/\s//g;      # dump white space
	$status =~ s/\s//g;  # dump white space
	# ID can have more than one entry.  Last one counts.

        if ($status =~ /REJECT/) { $color="red"; }
        elsif ($status =~ /ACCEPT/) { $color="green"; }
        elsif ($status =~ /SYSTEM/) { $color="blue"; }
        else { $color="black"; }

        $sta{$id} = "<font color=$color>$status</font>";
	}
    close(FH);

    #
    # Get all relevant data from Report_file.
    #

    local (%over, %man, %cman, %vol, %cvol, %con);
    local (%aa, $id, $mem, $id_mem, $overall, $rate, $conf);

    if (!open(FH, "$Report_file")) {
	print "cannot open file $Report_file\n";
	die;
        }

    local ($irs) = $/;            # save the default input record separator
    $/ = $DB_record_sep;

    while ($record = <FH>) {
        $record =~ s/$DB_record_sep//;   # dump rec sep at end of rec
	%aa = split($DB_field_sep, $record);
	$id = $aa{"paper_id"};
	$mem = $aa{"pc_member"};
	$overall = $aa{"overall"};
	$conf = $aa{"confidence"};
	$rate = $Omap{$overall};          # map rating to integer
	# A PC member can have more than one report on a paper,
	# and only the last one counts, so the following will
	# cause earlier ratings to be overwritten by the last one.
	$over{$id,$mem} = "$rate:$conf";  # shortcut; indexes really combined.
	}
    close(FH);
    $/ = $irs;            # restore the default input record seperator

    #
    # Process %over array, building arrays of mandatory and voluntary
    # ratings, indexed by paper ID.  Each entry is a list of ratings.
    # Also, build corresponding arrays of confidence levels.
    #

    foreach $id_mem (keys(%over)) {
	($id,$mem) = split($;, $id_mem);  # split index of assoc. array
	($rate,$conf) = split(':', $over{$id_mem});
	if ($assigned{$mem} =~ /\b$id\b/)
	    { $man{$id} .= ",$rate"; $cman{$id} .= ",$conf"; }
	else
	    { $vol{$id} .= ",$rate"; $cvol{$id} .= ",$conf"; }
        $man{$id} =~ s/^,//; $cman{$id} =~ s/^,//;
        $vol{$id} =~ s/^,//; $cvol{$id} =~ s/^,//;
        }

    #
    # Get all relevant data from Vote_file.
    #

    local (%vot, %acc, %rej, %abs);
    local ($vote);	

    if (!open(FH, "$Vote_file")) {
	print "cannot open file $Vote_file\n";
	die;
        }

    local ($irs) = $/;            # save the default input record separator
    $/ = $DB_record_sep;

    while ($record = <FH>) {
        $record =~ s/$DB_record_sep//;   # dump rec sep at end of rec
	%aa = split($DB_field_sep, $record);
	$id   = $aa{"paper_id"};
	$mem = $aa{"pc_member"};
	$vote  = $aa{"vote"};
	# A PC member can have more than one vote on a paper,
	# and only the last one counts, so the following will
	# cause earlier votes to be overwritten by the last one.
	$vot{$id,$mem} = $vote;  # perl shortcut; indexes really combined.
	}
    close(FH);
    $/ = $irs;            # restore the default input record seperator

    # 
    # Procvess %vot array, building arrays %acc, %rej, %abs, all indexed
    # by paper ID, containing the vote counts.
    #
    
    foreach $id_mem (keys(%vot)) {
	($id, $mem) = split($;, $id_mem);  # split index of assoc. array
	$vote = $vot{$id_mem};
	if ($vote eq "accept")
	    { $acc{$id} .= "A"; }
	elsif ($vote eq "reject")
	    { $rej{$id} .= "R"; }
	elsif ($vote eq "abstain")
	    { $abs{$id} .= "-"; }
        }

    #
    # From the Discuss_file, get the time of the most recent event
    # for each paper.
    #

    local ($date, %dat);

    if (!open(FH, "$Discuss_file")) {
	print "cannot open file $Discuss_file\n";
	die;
        }

    local ($irs) = $/;            # save the default input record separator
    $/ = $DB_record_sep;

    while ($record = <FH>) {
        $record =~ s/$DB_record_sep//;   # dump rec sep at end of rec
	%aa = split($DB_field_sep, $record);
	$id = $aa{"paper_id"};
	$dat{$id} = &shorten_date($aa{"date"});
        $dis{$id} = substr($aa{"pc_member"}, 0, 3);
	}
    close(FH);
    $/ = $irs;            # restore the default input record seperator

    #
    # Read the file of papers to get the titles and types.
    # Also compute scores.
    #

    local (%sco);
    local (@ids, @ordered_ids);
    local ($type, $title, $authors);

    if (!open(FH, $File_of_papers)) {
        print "Cannot open file $filename: $!";
        die;
        }

    while ($line = <FH>) {
        chop($line);
        ($id,$type,$title,$authors) = split(/\s+#\s+/, $line);

        $tit{$id} = substr($title, 0, $Short_title_length); # shorten title
        $typ{$id} = $type;

        # Calculate score for this paper.
        $sco{$id} = &score($man{$id}, $vol{$id}, $acc{$id}, $rej{$id});

        push(@ids, $id);
        }
    close(FH);

    #
    # If we are to print assigned papers only, delete the others.
    #

    if ($member_only) {
	local (@temp);
	foreach $id (@ids) {
	    if ($assigned{$pc_member} =~ /\b$id\b/) {
		push(@temp, $id);
	        }
	    }
	@ids = @temp;
        }

    #
    # Sort the list of IDs by score
    #

    @ordered_ids = sort by_score @ids;

    #
    # Now, we are ready to print the data for each paper.
    # We have a bunch of associative arrays, indexed by paper IDs.
    # 
    # $tit{$id} - shortened title
    # $typ{$id} - type
    # $man{$id} - overall scores (integers) from mandatory reports
    # $cman{$id} - corresponding confidence levels
    # $vol{$id} - overall scores (integers) from voluntary reports
    # $cvol{$id} - corresponding confidence levels
    # $acc{$id} - number of accept votes
    # $rej{$id} - number of reject votes
    # $sta{$id} - status of paper (undefined means undecided)
    # $dat{$id} - date of most recent discussion
    #
    # First, print an introduction.

    if ($member_only)
        { print "<H1>Current Status of Papers assigned to $pc_member.</H1>\n";}
    else
        { print "<H1>Current Status of All Papers</H1>\n";}

    print <<END_OF_INTRO;

<PRE>
Overall ratings are mapped to integers:
    Strong Accept   3        Strong Reject  -3
    Accept          2        Reject         -2
    Weak Accept     1        Weak Reject    -1
    No Opinion      0

discussion - Date (GMT) of most recent discussion
mandatory  - Overall ratings from mandatory reports
             [corresponding confidence levels are in brackets]
vol.       - Overall ratings from voluntary reports
accept     - Number of votes for accepting the paper (A)
reject     - Number of votes for rejecting the paper (R)
status (this is entered by the program chair):
    ACCEPT  : acceptance strongly proposed
    accept? : leaning to accept
    (empty) : undecided or borderline
    reject? : leaning to reject
    REJECT  : rejection strongly proposed
    ??????  : controversial (need discussion)
    !!!!!!  : need an expert review
    system? : leaning to system description instead
    SYSTEM  : system description strongly proposed
</PRE>

<P>
Lists are ordered by sum of mandatory ratings (then by ID).
<P>
END_OF_INTRO

    print "<TABLE border cellpadding=4>\n";

    #
    # If the pc_member is author of any papers,
    # print those papers without data.
    #

    if ($authored{$pc_member} =~ /\w/ && !$member_only) {

      print "<TR><TD COLSPAN=8>Data withheld on these papers,\n";
      print "because you are an author.</TD></TR>\n";
        
      foreach $id (@ordered_ids) {
        if ($authored{$pc_member} =~ /\b$id\b/) {
          print "<TR>\n";
          print "<TD><A HREF=\"$FTP_abstract_dir/$id.txt\">$id</A></TD>\n";
          print "<TD><A HREF=\"$FTP_ps_dir/$id.ps.Z\"><I>$tit{$id}</I></A>...</TD>\n";
          print "</TR>\n";
          }
        }
      }

    #
    # Print the data on regular papers (exclude if user is author).
    #

    print "<TR>\n";
    print "<TD COLSPAN=2>Regular Papers</TD>\n";
    print "<TD>discussion</TD>\n";
    print "<TD>mandatory</TD><TD>vol.</TD>\n";
    print "<TD>accept</TD><TD>reject</TD><TD align=center>status</TD>\n";
    print "</TR>\n";
        
    foreach $id (@ordered_ids) {
      if ($typ{$id} eq "regular" && !($authored{$pc_member} =~ /\b$id\b/)) {
        print "<TR>\n";
        print "<TD><A HREF=\"$FTP_abstract_dir/$id.txt\">$id</A></TD>\n";
        print "<TD><A HREF=\"$FTP_ps_dir/$id.ps.Z\"><I>$tit{$id}</I></A>...</TD>\n";
	$tmp = defined($dis{$id}) ? " ($dis{$id})" : "";
        print "<TD>$dat{$id}$tmp</TD>\n";
        print "<TD>$man{$id} [$cman{$id}]</TD>\n";

	$tmp = defined($vol{$id}) ? " [$cvol{$id}]" : "";
        print "<TD>$vol{$id}$tmp</TD>\n";
        print "<TD align=center>$acc{$id}</TD>\n";
        print "<TD align=center>$rej{$id}</TD>\n";
        print "<TD align=center>$sta{$id}</TD>\n";
        print "</TR>\n";
        }
      }

    #
    # Print the data on system papers (exclude if user is author).
    #

    print "<TR>\n";
    print "<TD COLSPAN=2>System Abstracts</TD>\n";
    print "<TD>discussion</TD>\n";
    print "<TD>mandatory</TD><TD>vol.</TD>\n";
    print "<TD>accept</TD><TD>reject</TD><TD align=center>status</TD>\n";
    print "</TR>\n";
        
    foreach $id (@ordered_ids) {
      if ($typ{$id} eq "system" && !($authored{$pc_member} =~ /\b$id\b/)) {
        print "<TR>\n";
        print "<TD><A HREF=\"$FTP_abstract_dir/$id.txt\">$id</A></TD>\n";
        print "<TD><A HREF=\"$FTP_ps_dir/$id.ps.Z\"><I>$tit{$id}</I></A>...</TD>\n";
	$tmp = defined($dis{$id}) ? " ($dis{$id})" : "";
        print "<TD>$dat{$id}$tmp</TD>\n";
        print "<TD>$man{$id} [$cman{$id}]</TD>\n";

	$tmp = defined($vol{$id}) ? " [$cvol{$id}]" : "";
        print "<TD>$vol{$id}$tmp</TD>\n";
        print "<TD align=center>$acc{$id}</TD>\n";
        print "<TD align=center>$rej{$id}</TD>\n";
        print "<TD align=center>$sta{$id}</TD>\n";
        print "</TR>\n";
        }
      }

    print "</TABLE>";


}  # summary_papers

