#
# chop_date - reurn date in GMT
#

sub chop_date {
    local ($date) = `/bin/date -u`;
    chop($date);
    return $date;
    }

#
# shorten_date - assume input is from chop_date.
#

sub shorten_date {
    local ($date) = @_;

#   Example: Mon Jan 27 09:43:03 GMT 1997

    $date =~ s/^(\D+) (\D+) (\d+) (\d\d:\d\d).*/\2 \3, \4/;
    return $date;
    }

#
# member
#

sub member {
    local ($object, @list) = @_;
    local ($element);

    foreach $element (@list) {
	return 1 if ($element eq $object);
        }
    return 0;
    }  # member

#
# Print the arguments, surrounded by the HTML <PRE> commands.
# This should NOT be called when already in <PRE> mode.
#

sub preprint {
    print "<PRE>\n";
    print  @_;
    print "</PRE>\n";
}  # preprint

#
# Get the value of a CGI parameter.  This should be called for
# values that are supposed to exist.  If the paramameter does not
# esist, or the value is empty, print an error message and die.
# 

sub get_cgi_value {
    local (*cgi_input, $key) = @_;
    local ($value);
	
    $value = $cgi_input{$key};
    if (! defined($value) || $value =~ /^\s*$/) {
	print "<H1>ERROR: $key not found.</H1>\n";
	die;
	}
    return $value;
}  # get_gci_value

#
# Print the CGI parameters.  This is meant for debugging.
#

sub print_cgi_input {
    local (*input) = @_;
    print "<PRE>\n";
    foreach $key (keys(%input)) {
	print "\n$key:\n $input{$key}\n";
	}
    print "</PRE>\n";
}  # print_cgi_input

#
# Print some system information.  This is meant for debugging.
#

sub print_sys_info {
    local ($key);
    print "<PRE>\n";
    print "Some system info.\n";
    print "real user id: $<.\n";
    print "effective user id: $>.\n";
    print "real group id: $(.\n";
    print "effective group id: $).\n";
    print "\nEnvironment:\n";
    foreach $key (keys(%ENV)) {
	print "$key: $ENV{$key}\n";
	}
    print "</PRE>\n";
}  # print_cgi_input

#
# get_paper - Given a paper ID, return the record, split into fields.
#

sub get_paper {
    local ($id) = @_;

    if (! open(FH, $File_of_papers)) {
	print "<H1>Cannot open file $File_of_papers.</H1>\n";
	die;
	}
    local (@papers) = <FH>;  # Read the whole file of papers at once.

    close(FH);
    local ($line) = grep(/^$id\s+#/, @papers);
    if (!defined($line)) {
	return undef;
	}
    else {
        chop($line);
	local (@fields) = split(/\s+#\s+/, $line);
	return @fields;
	}
}  # get_paper

#
# get_pc_data - given a pc member, get the record.  The record
# is split into fields, but the fields may contain whitespace at
# the beginning and end.
#

sub get_pc_data {
    local ($member) = @_;

    if (! open(FH, $PC_data_file)) {
	print "<H1>Cannot open file $PC_data_file.</H1>\n";
	die;
	}
    local (@data) = <FH>;  # Read the whole file of data at once.
    close(FH);
    local ($line) = grep(/^\s*$member\s*#/, @data);
    if (!defined($line)) {
	return undef;
	}
    else {
        chop($line);
	return split("#", $line);
	}
    }  # get_pc_data

#
# get_password
#

sub get_password {
    local ($member) = @_;

    local ($mem, $passwd) = &get_pc_data($member);
    return undef if (!defined($passwd));

    $passwd =~ s/\s//g;
    return $passwd;
    }  # get_password

#
# get_authored_papers
#

sub get_authored_papers {
    local ($member) = @_;

    local ($mem, $passwd, $authored_papers) = &get_pc_data($member);

    # The following is a special case of the split command.
    return split(" ",$authored_papers);
    }  # get_authored_papers

#
# get_assigned_papers
#

sub get_assigned_papers {
    local ($member) = @_;

    local ($mem,$passwd,$authored_papers,$assigned_papers) = &get_pc_data($member);

    # The following is a special case of the split command.
    return split(" ",$assigned_papers);
    }  # get_assigned_papers

#
# authored_paper - Did a given pc member author a given paper?
#

sub authored_paper {
    local ($member, $pid) = @_;
    local (@ids) = &get_authored_papers($member);
    return &member($pid, @ids);
    }  # authored_paper

#
# assigned_paper - Is a given pc member assigned to referee a given paper?
#

sub assigned_paper {
    local ($member, $pid) = @_;
    local (@ids) = &get_assigned_papers($member);
    return &member($pid, @ids);
    }  # assigned_paper

#
# Get the set of members assigned to referee a given paper.
#

sub get_assigned_members {
    local ($pid) = @_;
    local ($line, @fields, @papers, @members, $member);

    if (! open(FH, $PC_data_file)) {
	print "<H1>Cannot open file $PC_data_file.</H1>\n";
	die;
	}
    local (@data) = <FH>;  # Read the whole file of data at once.
    close(FH);

    foreach $line (@data) {
	@fields = split("#", $line);
        @papers = split(" ", $fields[3]);
	$member = $fields[0];
	$member =~ s/\s//g;
        if (&member($pid, @papers)) {
	    push(@members, $member);
	    }
	}

    return @members;

    }  # get_assigned_members

#
# Print a referee's report in plain text to stdout.
# This is the complete report, INCLUDING CONFIDENTIAL PC DATA.
#

sub print_ref_report {
    local ($fh, *report) = @_;

    local ($id,$ptype,$title,$authors) = &get_paper($report{"paper_id"});

    # Don't print paper type if it is a "regular" paper.

    local ($ptype_print) = ($ptype eq "regular" ? "" : " ($ptype)");

    print $fh <<END_OF_REPORT;

$Conference Referee's Report

Paper $report{"paper_id"}$ptype_print: $title
Author(s): $authors

Numerical values are in the range [1..4]  (4 is best).

Appropriate topic for $Conference: $report{"appropriateness"}
Significance:                  $report{"significance"}
Originality:                   $report{"originality"}
Presentation:                  $report{"presentation"}
Referee's confidence:          $report{"confidence"}
Overall rating:                $report{"overall"}

Comments for authors:

$report{"comments"}

-------------------------------
End of author section of report.

Comments for PC (optional):

$report{"comments_pc"}

PC Member: $report{"pc_member"}
Referee (if different from PC member): $report{"referee"}

End of report.
END_OF_REPORT

}  # print_ref_report

#
# Print a referee's report in plain text.
# This is the report to be sent to authors.
#

sub print_ref_report_for_authors {
    local ($fh, *report) = @_;

    local ($id,$ptype,$title,$authors) = &get_paper($report{"paper_id"});

    # Don't print paper type if it is a "regular" paper.

    local ($ptype_print) = ($ptype eq "regular" ? "" : " ($ptype)");

    $report{"comments"} =~ s/\r//g;   # get rid of ^M at ends of lines
    $report{"overall"} =~ s/No Opinion/Borderline/g;

    print $fh <<END_OF_REPORT;

-------------------------------------------------------------------
$Conference Referee's Report

Paper $report{"paper_id"}$ptype_print: $title
Author(s): $authors

Numerical values are in the range [1..4]  (4 is best).

Relevance to $Conference:   $report{"appropriateness"}
Significance:           $report{"significance"}
Originality:            $report{"originality"}
Presentation:           $report{"presentation"}
Referee's confidence:   $report{"confidence"}
Overall rating:         $report{"overall"}

Comments for authors:

$report{"comments"}

End of Report
END_OF_REPORT

}  # print_ref_report_for_authors

#
# get_overalls - Get overall ratings for a given paper.
# Separate into mandatory and voluntary.
#

sub get_overalls {
    local ($id, *mandatory, *voluntary) = @_;

    undef(%mandatory);	
    undef(%voluntary);	

    local (@reports, @assigned_members);
    local ($report, $pc_member, $overall);
    local (%aa);

    # Get set of PC members assigned to referee this paper.

    local (@assigned_members) = &get_assigned_members($id);

    # Get list of reports on this paper.

    @reports = &retrieve_events($Report_file, "paper_id", $id);

    # Get overall ratings.  If a member has filed more than one report
    # on this paper, the latest one will be counted, because the list
    # of reports is ordered by event ID (i.e., by time).

    foreach $report (@reports) {
	%aa = &record_to_assoc_array(*report);
	$pc_member = $aa{"pc_member"};
	if (&member($pc_member, @assigned_members)) {
	    $mandatory{$pc_member} = $aa{"overall"};
	    }
	else {
	    $voluntary{$pc_member} = $aa{"overall"};
	    }
        }

    # Note all members assigned to referee this paper who have
    # not yet filed a report.

    foreach $pc_member (@assigned_members) {
	if (! defined($mandatory{$pc_member})) {
	    $mandatory{$pc_member} = "(no report yet)";
	    }
        }

    }  # get_overalls

#
# get_votes - Get votes for a given paper.
#

sub get_votes {
    local ($id, *accepts, *rejects) = @_;

    undef (@accepts);	
    undef (@rejects);	

    local (@records);
    local (@reports, $report, $pc_member, $record);
    local (%aa, %count);

    # Get list of votes on this paper.

    @records = &retrieve_events($Vote_file, "paper_id", $id);

    # If a member has cast more than one vote
    # on this paper, the latest one will be counted, because the list
    # of votes is ordered by event ID (i.e., by time).

    foreach $record (@records) {
	%aa = &record_to_assoc_array(*record);
	$pc_member = $aa{"pc_member"};
	$count{$pc_member} = $aa{"vote"};
        }

    foreach $pc_member (keys(%count)) {
	if ($count{$pc_member} eq "accept") {push(@accepts, $pc_member);}
	elsif ($count{$pc_member} eq "reject") {push(@rejects, $pc_member);}
        }

    }  # get_votes

#
# log_access
#

sub log_access {
    local ($pc_member, $page, $message) = @_;

    local($date, $remote_host);

    local ($date) = &chop_date;
    local ($remote_host) = $ENV{'REMOTE_HOST'};
    if (open(FH, ">>$Access_file")) {
        print FH "$pc_member : $page : $message : $remote_host : $date.\n";
        close(FH);
        }
  
    }  # log_access

1;  # return true
